/*
 *  Single scale object detection with WaldBoost classifier.
 *
 *  The program takes video from camera, scans the image and displays the
 *  detections.
 */

// OpenCV
#include <cv.h>
#include <cxcore.h>
#include <highgui.h>
// STL
#include <stdio.h>
#include <assert.h>
#include <fstream>
#include <stack>
#include <vector>
// Detection engine
#include "engine.h" // Detection library
#include "classifier.h" // Classifier loader
#include "detector.h" // Classifier loader
#include "faces.h"      // Face detection classifier



using namespace std;

TRect rect(int _x, int _y, int _w, int _h)
{
    TRect r = {_x, _y, _w, _h};
    return r;
}



int main(int argc, char ** argv)
{
    //CvCapture * capture = cvCaptureFromFile(argv[1]);
    CvCapture * capture = cvCaptureFromCAM(0);

    if (!capture)
    {
        fprintf(stderr, "Error: No camera found\n");
        return 0;
    }

    //////////////////////
    
    TClassifier * c = 0;

    if (argc > 2)
    {
        c = loadClassifierXML(argv[2]);
    }

    if (!c) // if no classifier specified or incorrect xml is supplied, use default classifier
        c = &faceDetector;
    
    initClassifier(c);   // Initialization of a classifier
    c->threshold = 5;

    TRect results[1000]; // Vector for results
    TRect clustered[1000];
    int hist[1000];
    //////////////////////
    
    IplImage * gray = 0;
    Pyramid pyr(cvSize(320,240), 8, 3);

    cvNamedWindow("Image", 1);

    for (;;)
    {
        IplImage * inputFrame = cvQueryFrame(capture);
        if (!inputFrame)
            break;

        // Init images
        if (!gray)
        {
            gray = cvCreateImage(cvSize(inputFrame->width, inputFrame->height), IPL_DEPTH_8U, 1);
        }
        
        cvCvtColor(inputFrame, gray, CV_RGB2GRAY);
        cvSmooth(gray, gray);
        pyr.insert(gray);

        ///////////////////////////////////
        // Actual detection happens here.

        int n = abrDetectObjects(pyr, c, results, results+1000, hist);
        int g = abrClusterDetections(results, clustered, n, 0.5, 15, 640.0/320.0);
        
        ///////////////////////////////////
        // Show the image
        
        for (int i = 0; i < g; ++i)
        {
            TRect * r = clustered + i;
            cvRectangle(inputFrame, cvPoint(r->x,r->y), cvPoint(r->x+r->width,r->y+r->height), cvScalar(0,0,0, 0), 3, 8, 0);
            cvRectangle(inputFrame, cvPoint(r->x,r->y), cvPoint(r->x+r->width,r->y+r->height), cvScalar(0,255,0, 0), 1, 8, 0);
        }

        cvShowImage("Image", inputFrame);

        char key;
        if ((key = cvWaitKey(5)) == 'q') break;
        if (key == 'T') c->threshold += 0.5;
        if (key == 't') c->threshold -= 0.5;
    }
    
    ///////////////////////////////////
    // Release classiifier if needed (only when xml classifier is used)
    if (c != &faceDetector)
        releaseClassifier(&c);
    ///////////////////////////////////

    return 0;
}

